@@ -1235,7 +1235,7 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 		if (VALID_BACKEND(i))
 		{
 			/* set DB node id */
-			CONNECTION(backend, i)->db_node_id = i;
+			pool_set_db_node_id(CONNECTION(backend, i), i);
 
 			/* mark this is a backend connection */
 			CONNECTION(backend, i)->isbackend = 1;
@@ -1484,7 +1484,7 @@ void child_exit(int code)
  * create a persistent connection
  */
 POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
-	char *hostname, int port, char *dbname, char *user, char *password, bool retry)
+	int db_node_id, char *hostname, int port, char *dbname, char *user, char *password, bool retry)
 {
 	POOL_CONNECTION_POOL_SLOT *cp;
 	int fd;
@@ -1542,6 +1542,7 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	cp->con = pool_open(fd);
 	cp->closetime = 0;
 	cp->con->isbackend = 1;
+	pool_set_db_node_id(cp->con, db_node_id);
 	pool_ssl_negotiate_clientserver(cp->con);
 
 	/*
@@ -2082,7 +2083,7 @@ static void init_system_db_connection(void)
 			pool_error("Could not make persistent libpq system DB connection");
 		}
 
-		system_db_info->connection = make_persistent_db_connection(pool_config->system_db_hostname,
+		system_db_info->connection = make_persistent_db_connection(-1, pool_config->system_db_hostname,
 																   pool_config->system_db_port,
 																   pool_config->system_db_dbname,
 																   pool_config->system_db_user,
